﻿using DotNetCore.CAP;
using Hotline.Article;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.Orders;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using OracleInternal.SqlAndPlsqlParser.LocalParsing;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.FlowEngine;

public class RejectHandler : INotificationHandler<RejectNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;
    private readonly ILogger<RejectHandler> _logger;
    private readonly IRepository<OrderScreen> _orderScreenRepository;
    private readonly IRepository<Bulletin> _bulletinRepository;
    private readonly IRepository<Circular> _circularRepository;

	public RejectHandler(
        IOrderDomainService orderDomainService,
        IOrderRepository orderRepository,
        ICapPublisher capPublisher,
        IMapper mapper,
        ILogger<RejectHandler> logger,
        IRepository<OrderScreen> orderScreenRepository,
        IRepository<Bulletin> bulletinRepository,
        IRepository<Circular> circularRepository)
    {
        _orderDomainService = orderDomainService;
        _orderRepository = orderRepository;
        _capPublisher = capPublisher;
        _mapper = mapper;
        _logger = logger;
        _orderScreenRepository = orderScreenRepository;
        _bulletinRepository = bulletinRepository;
        _circularRepository = circularRepository;
	}

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(RejectNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;
        var data = notification.Dto;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderDelay:

                break;
            case WorkflowModuleConsts.OrderScreen:
	            var screen = await _orderScreenRepository.GetAsync(workflow.ExternalId, cancellationToken);
	            if (screen != null)
	            {
		            screen.Status = EScreenStatus.Refuse;
		            screen.ReplyContent = workflow.ActualOpinion;
		            await _orderScreenRepository.UpdateAsync(screen, cancellationToken);
	            }
	            break;
            case WorkflowModuleConsts.BulletinApply:
                var bulletin = await _bulletinRepository.GetAsync(workflow.ExternalId, cancellationToken);
                if (bulletin != null)
                {
                    bulletin.BulletinState = Share.Enums.Article.EBulletinState.ReviewNoPass;
                    await _bulletinRepository.UpdateAsync(bulletin, cancellationToken);
                }
                break;
            case WorkflowModuleConsts.CircularApply:
                var circular = await _circularRepository.GetAsync(workflow.ExternalId, cancellationToken);
                if (circular != null)
                {
                    circular.CircularState = Share.Enums.Article.ECircularState.ReviewNoPass;
                    await _circularRepository.UpdateAsync(circular, cancellationToken);
                }
                break;
		}
    }
}